#!/usr/bin/perl -w
use warnings;
use strict;

use POSIX qw/floor/;
use Pod::Usage;
use Getopt::Long;
use Spreadsheet::WriteExcel;
use Switch;

my $val='time_avg';
my $excel_file_name='temp.xls';
my $directory=" ";     #'fca_stats/';

my $excel_sheet_size = 100;

my $help=0;
my $man=0;
my $type=0;
my @xml_files;


my %time_table = ( "time_total",0,
					"fca_time_total",0,
					"allreduce",0,
					"barrier",0,
					"bcast", 0,
					"reduce",0,
					"allgather",0,
					"allgatherv",0);


GetOptions( 'man'=> \$man,
	'f=s'=> \$excel_file_name,
	'd=s'=> \$directory,
	'val=s'=> \$val,
	'help|?' => \$help)
	or pod2usage( "Try '$0 --help' for more information." );
if ($man){
	pod2usage(-verbose => 2);
}
if ($help){
	pod2usage(1);
}

if ($directory eq " "){
	pod2usage( "you need to set xml directory -d 'directory-name'." );
}else{
	$directory="$directory/";
}

#------array xml file------input check and sorting--------#
my $tempxml="";
my $x=open_all_xml_files($directory);
$tempxml="$tempxml$x";

@xml_files=split(" ",$tempxml);
@xml_files=sort cmp_string @xml_files;


if ($#xml_files<1){
	pod2usage( "no xml file found." );
}

my $workbook  = Spreadsheet::WriteExcel->new($excel_file_name);
my $worksheet = $workbook->add_worksheet();

my $format = $workbook->add_format();
$format->set_align('center');
my $format1 = $workbook->add_format();
$format1->set_align('center');
$format1->set_bold();
my $format2 = $workbook->add_format();
$format2->set_align('center');
$format2->set_bold();
$format2->set_bg_color('grey');


my %attrs;
my $comm_id=0;
my $comm_size=0;
my $index_row=0;


# to make sure the number of nodes is smaller then excel limit

my $xml_number_end =  min($#xml_files,$excel_sheet_size)+1;
my $xml_number_start = 0;


while ($xml_number_start < $xml_number_end){

	foreach my $type ("allreduce",'barrier','bcast','reduce','allgather','allgatherv'){#,'gather','gatherv'){


		my %table;	# create table
		my $rank="";
		$time_table{'time_total'}=0;

		for (my $i=$xml_number_start; $i<$xml_number_end; $i++){
			my $XMLFILE;
			open($XMLFILE , "<", $xml_files[$i]) or die "Error: Unable to open $xml_files[$i].!\n";
			while (<$XMLFILE>){

				my $line = $_;
				if($line =~ /^(.*)<fca (.*)>/) {
					%attrs = parse_attr($2);
					$rank=$rank.' '.$attrs{'rank_id'};
					$time_table{'time_total'}+=$attrs{'time_total'};
				}elsif($line =~ /^(.*)<comm (.*)>/) {
					%attrs = parse_attr($2);
					$comm_id=$attrs{'id'};
					$comm_size=$attrs{'size'};
				}elsif($line =~ /^(.*)<$type (.*)>/) {
					%attrs = parse_attr($2);
		
					my $key="";
					if (($type eq "allreduce") || ($type eq "reduce")){
						$time_table{$type}+=$attrs{'time_total'};
						my @temp=split(';',$attrs{'log_buf_size'});  my $bytes=2**$temp[1];
						$key="$comm_id $comm_size $attrs{'count'} $bytes $attrs{'op'} $attrs{'dtype'}";
						my $attr_val=$attrs{"$val"};
						if ($table{$key}){
							my $temp_key=$table{$key};
							$temp_key= "$temp_key $attr_val";
							$table{$key}=$temp_key;
						}else{
							$table{$key}=$attr_val;
						}				
					}elsif (($type eq "bcast") || ($type eq "allgather")|| ($type eq "allgatherv") || ($type eq "gather") || ($type eq "gatherv")){
						$time_table{$type}+=$attrs{'time_total'};
						my @temp=split(';',$attrs{'log_buf_size'}); my $bytes=2**$temp[1];
						$key="$comm_id $comm_size $attrs{'count'} $bytes";
						my $attr_val=$attrs{"$val"};
						if ($table{$key}){
							my $temp_key=$table{$key};
							$temp_key= "$temp_key $attr_val";
							$table{$key}=$temp_key;
						}else{
							$table{$key}=$attr_val;
						}	
					}elsif ($type eq "barrier"){
						$time_table{$type}+=$attrs{'time_total'};
						#print $attrs{'time_total'}."\n";
						$key="$comm_id $comm_size $attrs{'count'}";
						my $attr_val=$attrs{"$val"};
						if ($table{$key}){
							my $temp_key=$table{$key};
							$temp_key= "$temp_key $attr_val";
							$table{$key}=$temp_key;
						}else{
							$table{$key}=$attr_val;
						}	
					}
				}	
			}#end while xml
		}#end while for
		$worksheet->write($index_row, 0,  "$type",$format1);
		$index_row++;
		$worksheet->write($index_row, 0,  "comm/rank",$format2);
		$worksheet->write($index_row, 1 ,  "comm_size",$format2);
		$worksheet->write($index_row, 2 ,  "count",$format2);
		if (($type eq "allreduce") || ($type eq "reduce")){
			$worksheet->write($index_row, 3 ,  "bytes",$format2);
			$worksheet->write($index_row, 4 ,  "op",$format2);
			$worksheet->write($index_row, 5 ,  "dtype",$format2);
			$worksheet->write($index_row-1, 6,  "$val",$format1);
		}elsif (($type eq "bcast")||($type eq "allgather") || ($type eq "allgatherv") || ($type eq "gather") || ($type eq "gatherv")){
			$worksheet->write($index_row, 3 ,  "bytes",$format2);
			$worksheet->write($index_row-1, 4,  "$val",$format1);
		}else{
			$worksheet->write($index_row-1, 3,  "$val",$format1);
		}
		my $flag=0;
		while ((my $key, my $value) = each(%table)){
			$index_row++;
			my @keysplit=split(' ',$key);
			my @valuesplit=split(' ',$table{$key});
			my @ranksplit=split(' ',$rank);
			my $index_col=0;
			foreach my $temp_key (@keysplit){
				$worksheet->write($index_row,$index_col,$temp_key,$format);
				$index_col++;
			}
			if (!$flag){
				my $temp_col=$index_col;
				foreach my $temp_key (@ranksplit){
					$worksheet->write($index_row-1,$temp_col,$temp_key,$format2);
					$temp_col++;
				}
				$flag=1;
			}
			foreach my $temp_key (@valuesplit){
				$worksheet->write($index_row,$index_col,$temp_key,$format);
				$index_col++;
			}
		}	
		$index_row++;
		$index_row++;
	}


	$index_row+=9;
	$xml_number_start = $xml_number_end;
	$xml_number_end = $xml_number_end + $excel_sheet_size;
	$xml_number_end = min($xml_number_end,($#xml_files+1));
}
$time_table{'fca_time_total'} = $time_table{'allreduce'}+$time_table{'barrier'}+$time_table{'bcast'}+$time_table{'reduce'}+$time_table{'allgather'}+$time_table{'allgatherv'};
$index_row+=1;
my $temp_time = set_time($time_table{"time_total"});
$worksheet->write($index_row,1,"total time",$format);
$worksheet->write($index_row,2,$temp_time,$format);
$worksheet->write($index_row+1,1,"total fca time",$format);
$temp_time = set_time($time_table{"fca_time_total"});
$worksheet->write($index_row+1,2,$temp_time,$format);
my $percent = floor(($time_table{"fca_time_total"}/$time_table{"time_total"})*100);
$worksheet->write($index_row+2,1,"percent",$format);
$worksheet->write($index_row+2,2,"$percent%",$format);

$worksheet->write($index_row,6,"allreduce",$format);
$temp_time = set_time($time_table{"allreduce"});
$worksheet->write($index_row+2,6,$temp_time,$format);
$percent = floor(($time_table{"allreduce"}/$time_table{"fca_time_total"})*100);
$worksheet->write($index_row+1,6,"$percent%",$format);
$worksheet->write($index_row,7,"barrier",$format);
$percent = floor(($time_table{"barrier"}/$time_table{"fca_time_total"})*100);
$worksheet->write($index_row+1,7,"$percent%",$format);
$temp_time = set_time($time_table{"barrier"});
$worksheet->write($index_row+2,7,$temp_time,$format);
$worksheet->write($index_row,8,"bcast",$format);
$percent = floor(($time_table{"bcast"}/$time_table{"fca_time_total"})*100);
$worksheet->write($index_row+1,8,"$percent%",$format);
$temp_time = set_time($time_table{"bcast"});
$worksheet->write($index_row+2,8,$temp_time,$format);
$worksheet->write($index_row,9,"reduce",$format);
$percent = floor(($time_table{"reduce"}/$time_table{"fca_time_total"})*100);
$worksheet->write($index_row+1,9,"$percent%",$format);
$temp_time = set_time($time_table{"reduce"});
$worksheet->write($index_row+2,9,$temp_time,$format);
$worksheet->write($index_row,10,"allgather",$format);
$percent = floor(($time_table{"allgather"}/$time_table{"fca_time_total"})*100);
$worksheet->write($index_row+1,10,"$percent%",$format);
$temp_time = set_time($time_table{"allgather"});
$worksheet->write($index_row+2,10,$temp_time,$format);
$worksheet->write($index_row,11,"allgatherv",$format);
$percent = floor(($time_table{"allgatherv"}/$time_table{"fca_time_total"})*100);
$worksheet->write($index_row+1,11,"$percent%",$format);
$temp_time = set_time($time_table{"allgatherv"});
$worksheet->write($index_row+2,11,$temp_time,$format);



sub set_time{
	my $the_time= $_[0]/100;
	my %array = ('hour',0,
		         'minute',0,
				 'second',0);
	$the_time= floor($the_time);
	$array{'minute'} = floor($the_time/60);
	$array{'second'} = $the_time-($array{'minute'}*60);
	$array{'hour'} = floor($array{'minute'}/60);
	$array{'minute'} = $array{'minute'}-($array{'hour'}*60);
	my $print_time = "$array{'hour'}::$array{'minute'}::$array{'second'}";

	return $print_time;
}

sub open_all_xml_files {
	my $tempxml ="";
	my $DIR;
	my $directory=$_[0];
	opendir (DIR, $directory ) or die "Couldn't open directory, $!";
	
	while (my $file = readdir DIR)
	{
		if ($file=~'xml'){
			my $files="$directory$file";
			if ($directory eq '.'){
				$files="$file";
			}else{
				$files="$directory$file";
			}
			open(my $XMLFILE , "<", $files) or die("Could not open $file file.");
			close($XMLFILE);
			$tempxml="$tempxml$files ";
		}
	}
	closedir DIR;
	return $tempxml;
}

sub parse_attr
{
	my ($line) = @_;
	my $k = $line;

	#print "orig: $line\n";
	$k =~ s/="/ "/g;
	$k =~ s/"//g;

	my $cmd = 'my %r = qw(' . $k . ');';
	my %res = eval $cmd;

	%res;
}

sub cmp_string
{
	my @temp1=split('\.',$a);
	my @temp2=split('\.',$b);
	if ($temp1[$#temp1-1]>$temp2[$#temp2-1]){
		return 1;
	}elsif ($temp1[$#temp1-1]==$temp2[$#temp2-1]){
		return 0;
	}
	return -1;
}

sub min
{
	if ($_[1]<$_[0]){
		return $_[1];
	}
	return $_[0];
}



__END__

=head1 NAME

nodes - search switch elements.

=head1 SYNOPSIS

   Options:

	-? or -help               Display help.
	-man                      About the script.
	-d "directory name"       set xml directory.
	-val "value"              Change xml value. (default "time_avg")
	-f "excel file name"      Change the output file name. (default - 'temp.xls')

	to run-  fca_stat_analyze.pl -f "test.xls" -val "time_avg" -d "fca_stat"

=head1 DESCRIPTION

This package analyze xml files and generate an excel file from the data

=head1 OPTIONS

=over 8

=item B<-help>

Display a brief help message.

=item B<-man>

Display the manual page.

=item B<-f>

Change xls output name.

default is temp.xls.

=item B<-d>

Change xml input directory.

default '.'

=item B<-val>

Change xml value.

default "time_avg"

=back

